#include <stdio.h>
#include <cuda.h>

/*
 * Programa para imprimir por pantalla la información de CUDA de los distintos dispositivos conectados.
 */

#define CUDA_OK cudaSuccess

void imprimirErrorCUDA(char* durante ,cudaError_t error){
  fprintf(stderr,"CUDA Error durante: %s \nMensaje: %s\n",cudaGetErrorString(error));
}

int main(void){
  ///////////////////////////////
  //////// INFO DE CUDA /////////
  ///////////////////////////////
  cudaError_t error_cuda = CUDA_OK;
  //Obtener info de todos los dispositivos
  int device_count =0;
  if((error_cuda = cudaGetDeviceCount(&device_count))!=CUDA_OK){
    imprimirErrorCUDA("Obteniendo cuenta de dispositivos",error_cuda);
    return -1;
  }
  fprintf(stderr,"Hay %i dispositivos\n",device_count);
  
  cudaDeviceProp * propiedades = (cudaDeviceProp*) malloc(device_count * sizeof(cudaDeviceProp));

  //Obtener info de cada  dispositivo
  int i = 0;
  for (i=0;i<device_count;i++){
    if((error_cuda = cudaGetDeviceProperties(&propiedades[i],i))!=CUDA_OK){
      imprimirErrorCUDA("Obteniendo info de dispositivo",error_cuda);
      continue;
    }
    fprintf(stderr,"////////////////////////////////////////////\n");
    fprintf(stderr,"Info de dispositivo %i: %s obtenida correctamente:\n",i,propiedades[i].name);
    fprintf(stderr,"Tamanio warp: %i\n",propiedades[i].warpSize);     
    fprintf(stderr,"Memoria total: %lli Mbytes (size_t)\n",propiedades[i].totalGlobalMem >> 20);     
    fprintf(stderr,"Multiprocesadores: %i\n",propiedades[i].multiProcessorCount);     
    fprintf(stderr,"Max threads por MP: %i\n",propiedades[i].maxThreadsPerMultiProcessor);     
    fprintf(stderr,"Max threads por bloque: %i\n",propiedades[i].maxThreadsPerBlock);     
    fprintf(stderr,"Dimensiones max del bloque: %i %i %i\n",propiedades[i].maxThreadsDim[0],propiedades[i].maxThreadsDim[1],propiedades[i].maxThreadsDim[2]);     
    fprintf(stderr,"Dimensiones max del grid: %i %i %i\n",propiedades[i].maxGridSize[0], propiedades[i].maxGridSize[1],propiedades[i].maxGridSize[2]);     
    fprintf(stderr,"DeviceOverlap (varios streams): %i\n",propiedades[i].deviceOverlap);     
    fprintf(stderr,"////////////////////////////////////////////\n");
  }

  return 0;
}
